@@ -2,30 +2,29 @@
 #' Model-based Gradient Boosting for Functional Response
 #' 
 #' Gradient boosting for optimizing arbitrary loss functions, where component-wise models 
-#' are utilized as base-learners in the case of functional response.
+#' are utilized as base-learners in the case of functional response. 
+#' Scalar responses are treated as the special case where each functional response has 
+#' only one observation. 
 #' This function is a wrapper for \code{mboost}'s \code{\link[mboost]{mboost}} and its 
 #' siblings to fit models of the general form 
-#' \cr \eqn{E(Y_i(t)) = g(\mu(t) + \int X_i(s)\beta(s,t)ds + f(z_{1i}, t) + 
-#' f(z_{2i}) + z_{3i} \beta_3(t) + \dots }\cr
-#' with a functional (but not necessarily continuous) response \eqn{Y(t)}, response function \eqn{g},
-#' (optional) smooth intercept \eqn{\mu(t)}, (multiple) functional covariates \eqn{X(t)} 
-#' and scalar covariates \eqn{z_1}, \eqn{z_2}, etc. 
-#' 
-#' @details The functional response and functional covariates have to be
-#' supplied as n by <no. of evaluations> matrices, i.e. each row is one
-#' functional observation. For the model fit the matrix of the functional
-#' response evaluations \eqn{Y_i(t)} are stacked into one long vector.
-#' If the functional response is supplied as a vector in long format,  
-#' the argument \code{id} has to be specified.
+#' \cr \eqn{xi(Y_i(t)) = \sum_j h_j(x_i, t)} \cr
+#' with a functional (but not necessarily continuous) response \eqn{Y(t)}, 
+#' transformation function \eqn{xi}, e.g. the expectation, the median or some quantile, 
+#' and partial effects \eqn{h_j(x_i, t)} depending on covariates 
+#' and the current index of the response. 
+#' Possible effects are e.g. a smooth intercept \eqn{\mu(t)}, 
+#' effects of functional covariates \eqn{\int X_i(s)\beta(s,t)ds}, 
+#' smooth and linear effects of scalar covariates \eqn{f(z_{i})}, \eqn{z_{i} \beta_3(t)}. 
 #' 
 #' @param formula a symbolic description of the model to be fit.
 #' @param timeformula formula for the expansion over the index of the response. 
-#' For a functional response \eqn{Y_i(t)} typically ~bbs(t) to obtain a smooth 
-#' expansion of the effects along t. In the limiting case that \eqn{Y_i} is a scalar response
+#' For a functional response \eqn{Y_i(t)} typically \code{~bbs(t)} to obtain a smooth 
+#' expansion of the effects along \code{t}. In the limiting case that \eqn{Y_i} is a scalar response
 #' use \code{~bols(1)}, which sets up a base-learner for the scalar 1.
 #' @param id defaults to NULL which means that the response is a matrix with a regular time. 
 #' If the response is given in long format for irregular observations, \code{id} 
-#' contains the information which observations belong together. id should contain numbers 1, 2, 3, ...
+#' contains the information which observations belong together. 
+#' \code{id} should contain numbers 1, 2, 3, ...
 #' @param numInt integration scheme for the integration of the loss function.
 #' One of \code{c("equal", "Riemann")} meaning equal weights of 1 or 
 #' trapezoidal Riemann weights.
@@ -38,12 +37,27 @@
 #' \code{length(weights)} has to be \code{nrow(response)}*\code{ncol(response)}
 #' per default weights is constantly 1. 
 #' @param offset_control parameters for the calculation of the offset, 
-#' defaults to \code{offset_control(k_min=20, silent=TRUE)}.  
+#' defaults to \code{o_control(k_min=20, silent=TRUE)}, see \code{\link{o_control}}.  
 #' @param offset a numeric vector to be used as offset over the index of the response (optional).
 #' If no offset is specified, per default a smooth time-specific offset is calculated and used 
 #' within the model fit. If you do not want to use an offset you can set \code{offset=0}.
+#' @param check0 logical, for response observed on a common grid, 
+#' check the fitted effects for the sum-to-zero constraint 
+#' \eqn{h_j(x_i)(t) = 0} for all \eqn{t} and give warning if it is not fulfilled. Defaults to TRUE. 
 #' @param ... additional arguments passed to \code{\link[mboost]{mboost}}, 
-#' including \code{offset}, \code{family} and \code{control}.
+#' including, \code{family} and \code{control}.
+#' 
+#' @details The functional response and functional covariates have to be
+#' supplied as n by <no. of evaluations> matrices, i.e. each row is one
+#' functional observation. For the model fit the matrix of the functional
+#' response evaluations \eqn{Y_i(t)} are stacked into one long vector. 
+#' If it is possible to write the model as a generalized linear array model, 
+#' the array structure is used as efficient implementation, 
+#' see \code{\link[mboost]{mboost}}.
+#' If the response is observed irregularly or sparse it can be supplied  
+#' as a vector in long format. In that case the argument \code{id} has  
+#' to be specified as integers 1, 2, 3, ... to define which observation
+#' belongs to which curve. 
 #' 
 #' @return on object of class \code{FDboost} that inherits from \code{mboost}.
 #' Special \code{\link{predict.FDboost}}, \code{\link{coef.FDboost}} and 
@@ -66,13 +80,20 @@
 #' \item{formulaMboost}{the formula with which \code{mboost} was called within \code{FDboost}}
 #' 
 #' @author Sarah Brockhaus, Torsten Hothorn
+#' 
 #' @seealso \code{\link[mboost]{mboost}} for the help of the wrapped function in 
 #' package mboost.  
 #' See \code{\link[FDboost]{bsignal}} and \code{\link[FDboost]{bbsc}} 
 #' for possible base-learners
 #' 
 #' @keywords models, nonlinear 
-#' @examples  
+#' 
+#' @references 
+#' Brockhaus, S., Scheipl, F., Hothorn, T. and Greven, S. (2015). 
+#' The Functional Linear Array Model. Statistical Modelling, in press.
+#' 
+#' @examples 
+#' ## Example for function-on-scalar-regression 
 #' data("viscosity", package = "FDboost") 
 #' ## set time-interval that should be modeled
 #' interval <- "101"
@@ -83,21 +104,56 @@
 #' viscosity$time <- viscosity$timeAll[1:end]
 #' # with(viscosity, funplot(time, vis, pch=16, cex=0.2))
 #' 
-#' ## fit median regression model with 200 boosting iterations,
-#' ## step-length 0.2 and smooth time-specific offset
-#' mod <- FDboost(vis ~ 1 + bols(T_C) + bols(T_A),
+#' ## fit median regression model with 100 boosting iterations,
+#' ## step-length 0.4 and smooth time-specific offset
+#' ## the factors are in effect coding -1, 1 for the levels
+#' mod1 <- FDboost(vis ~ 1 + bols(T_C, contrasts.arg = "contr.sum", intercept=FALSE) 
+#'                + bols(T_A, contrasts.arg = "contr.sum", intercept=FALSE),
 #'                timeformula=~bbs(time, lambda=100),
 #'                numInt="Riemann", family=QuantReg(),
 #'                offset=NULL, offset_control = o_control(k_min = 9),
 #'                data=viscosity, control=boost_control(mstop = 100, nu = 0.4))
-#' summary(mod)
+#' summary(mod1)
+#' ## plot(mod1)
+#' 
+#' ## Example for scalar-on-function-regression 
+#' data("fuelSubset", package = "FDboost")
+#' 
+#' mod2 <- FDboost(heatan ~ bsignal(UVVIS, uvvis.lambda, knots=40, df=4, check.ident=FALSE) 
+#'                + bsignal(NIR, nir.lambda, knots=40, df=4, check.ident=FALSE), 
+#'                timeformula=~bols(1), data=fuelSubset) 
+#' summary(mod2) 
+#' ## plot(mod2)
+#' 
+#' ## Example for function-on-function-regression 
+#' if(require(fda)){
+#'   data("CanadianWeather", package="fda")
+#'   CanadianWeather$l10precip <- t(log(CanadianWeather$monthlyPrecip))
+#'   CanadianWeather$temp <- t(CanadianWeather$monthlyTemp)
+#'   CanadianWeather$region <- factor(CanadianWeather$region)
+#'   CanadianWeather$month.s <- CanadianWeather$month.t <- 1:12
+#'   
+#'   ## fit model with cyclic splines over the year
+#'   mod3 <- FDboost(l10precip ~ bols(region, df=2.5, contrasts.arg = "contr.dummy") 
+#'                  + bsignal(temp, month.s, knots=11, cyclic=TRUE, 
+#'                            df=2.5, boundary.knots=c(0.5,12.5), check.ident=FALSE), 
+#'                  timeformula=~bbs(month.t, knots=11, cyclic=TRUE, 
+#'                                   df=3, boundary.knots=c(0.5,12.5)), 
+#'                                   offset=0, offset_control = o_control(k_min=5), 
+#'                   data=CanadianWeather)   
+#'  summary(mod3)
+#'  ## plot(mod3, pers=TRUE)
+#' }
+#' 
+#' 
 #'                 
 #' @export
 #' @import mboost Matrix 
 #' @importFrom splines bs splineDesign
 #' @importFrom mgcv gam s
 #' @importFrom zoo na.locf
-#' @importFrom nnls nnls
+#' @importFrom MASS Null
+#' @importFrom parallel mclapply
 FDboost <- function(formula,          ### response ~ xvars
                     timeformula,      ### time
                     id=NULL,          ### id variable if response in long format
@@ -105,12 +161,33 @@ FDboost <- function(formula,          ### response ~ xvars
                     data,             ### list of response, time, xvars
                     weights = NULL,   ### optional
                     offset = NULL,    ### optional
-                    offset_control = o_control(),
+                    offset_control = o_control(), ### optional specification of offset model
+                    check0 = FALSE,    ### check sum-to-zero-constraint of the fitted effects?
                     #offset_control = list(k_min=20, silent=TRUE),
                     ...)              ### goes directly to mboost
 {
   dots <- list(...)
 
+  ## insert the id variable into the formula, to treat is like the other variables
+  if(!is.null(id)){
+    stopifnot(class(id)=="formula")
+    tf <- terms.formula(formula, specials=c("c"))
+    trmstrings <- attr(tf, "term.labels")
+    if(length(trmstrings)>0) trmstrings <- paste(substr(trmstrings, 1 , nchar(trmstrings)-1), ", index=", id[2],")", sep="")
+    xpart <- paste(as.vector(trmstrings), collapse = " + ")
+    if(xpart!=""){
+      if(any(substr(tf[[3]], 1, 1)=="1")) xpart <- paste0("1 + ", xpart)
+    }else{
+      xpart <- 1
+    }
+    formula <- as.formula(paste(tf[[2]], " ~ ", xpart))
+    #print(formula)
+    nameid <- paste(id[2])
+    id <- data[[nameid]]
+  }else{
+    nameid <- NULL
+  }
+  
   ### save formula of FDboost
   formulaFDboost <- formula
   
@@ -148,8 +225,9 @@ FDboost <- function(formula,          ### response ~ xvars
   
   ### extract covariates
   # data <- as.data.frame(data)
-  if(length(all.vars(formula)) > 1){
-    data <- data[all.vars(formula)[-1]]    
+  allCovs <- unique(c(nameid, all.vars(formula)))
+  if(length(allCovs) > 1){
+    data <- data[allCovs[!allCovs %in% c(yname, nameyind)] ]    
   }else data <- list(NULL)  # <SB> intercept-model without covariates
       
   ### get covariates that are modeled constant over time
@@ -186,6 +264,9 @@ FDboost <- function(formula,          ### response ~ xvars
     # check length of response and its time and index
     stopifnot(length(response)==length(time) & length(response)==length(id))
     
+    if(any(is.na(response))) warning("For non-grid observations the response should not contain missing values.")
+    if( !all(sort(unique(id))==1:length(unique(id))) ) warning("id should be integers 1, 2, 3, ...")
+    
     nr <- length(response) # total number of observations
     nc <- length(unique(id)) # number of trajectories
     dresponse <- as.vector(response) # column-wise stacking of response  
@@ -198,9 +279,9 @@ FDboost <- function(formula,          ### response ~ xvars
   ### variable to fit smooth intercept
   assign("ONEx", rep(1.0, nobs))
   
-#   ### FIXME: plausibility check: 
-#   # for constrained effects in the formula the model should include an intercept
-#   grep("bbsc", trmstrings) + grep("brandomc", trmstrings)
+  #   ### FIXME: plausibility check: 
+  #   # for constrained effects in the formula the model should include an intercept
+  #   grep("bbsc", trmstrings) + grep("brandomc", trmstrings)
   
   ### compose mboost formula
   cfm <- paste(deparse(formula), collapse = "") 
@@ -237,13 +318,14 @@ FDboost <- function(formula,          ### response ~ xvars
       cfm <- paste("bols(ONEtime, intercept = FALSE, df = ", c_df ,")")
     }
   }
-  
+
   # expand formula as Kronecker or tensor product 
   if(is.null(id)){
     tmp <- outer(xfm, tfm, function(x, y) paste(x, y, sep = "%O%"))
   }else{
     # expand the bl accoridng to id
-    xfm <- paste(substr(xfm, 1 , nchar(xfm)-1), ", index=id)", sep="")
+    if(grepl("ONEx", xfm[[1]])) xfm[[1]] <- paste(substr(xfm[[1]], 1 , nchar(xfm[[1]])-1), ", index=",nameid,")", sep="")
+    xfm <- paste(substr(xfm, 1 , nchar(xfm)-1), ")", sep="") # , index=id is done in the beginning
     tmp <- outer(xfm, tfm, function(x, y) paste(x, y, sep = "%X%"))
   }
 
@@ -285,7 +367,8 @@ FDboost <- function(formula,          ### response ~ xvars
   if(is.null(id)){
     
     ## per default add smooth time-specific offset 
-    if(is.null(offset) & dim(response)[2]>1){
+    if(is.null(offset) && dim(response)[2] > 1 && 
+         any(colMeans(response) > .Machine$double.eps *10^10)){
       message("Use a smooth offset.") 
       ### <FixMe> is the use of family@offset correct?
       #meanY <- colMeans(response, na.rm=TRUE)
@@ -314,13 +397,16 @@ FDboost <- function(formula,          ### response ~ xvars
       for(i in 1:nc){
         try(meanY[i] <- offsetFun(responseInter[,i], 1*!is.na(responseInter[,i]) ), silent=TRUE)
       }
+      # meanY <- sapply(1:nc, function(i) offsetFun(responseInter[,i], 1*!is.na(responseInter[,i])))
+      
       if( is.null(meanY) ||  any(is.na(meanY)) ){
         warning("Mean offset cannot be computed by family@offset(). Use a weighted mean instead.")
         meanY <- c()
         for(i in 1:nc){
           meanY[i] <- Gaussian()@offset(responseInter[,i], 1*!is.na(responseInter[,i]) )
         }
-      }   
+      }
+      rm(responseInter, meanNA)
       ### <FixMe> is the computation of k ok? 
       if(!offset_control$cyclic){
         modOffset <- try( gam(meanY ~ s(time, bs="ad", 
@@ -347,8 +433,10 @@ FDboost <- function(formula,          ### response ~ xvars
         ret      
       } 
       offset <- as.vector(matrix(offsetVec, ncol=ncol(response), nrow = nrow(response), byrow=TRUE))    
-    }else{    
-      if(dim(response)[2]==1){ ### scalar response
+    }else{ 
+      ### scalar response or mean-centered response -> one constant offset value is used
+      if(dim(response)[2]==1 | 
+           all(colMeans(response) < .Machine$double.eps *10^10)){ 
         offsetVec <- offset
         offset <- offset # use one constant offset in mboost()
         predictOffset <- function(time) if(is.null(offset)) 0 else offset
@@ -428,8 +516,6 @@ FDboost <- function(formula,          ### response ~ xvars
 
   }
   
-  #browser()
-  
   if (length(data) > 0) {
     ### mboost isn't happy with nrow(data) == 0
     ret <- mboost(fm, data = data, weights = w, offset=offset, ...) 
@@ -437,6 +523,30 @@ FDboost <- function(formula,          ### response ~ xvars
     ret <- mboost(fm, weights = w, offset=offset, ...)
   }
   
+  # check sum-to-zero constraints for the fitted effects
+  # for models with more than one effect and a regular response
+  # not for scalar response
+  if(check0 && length(ret$baselearner)>1 && is.null(id) && dim(response)[2]!=1){
+    
+    # do not check the smooth intercept
+    if(any( gsub(" ", "", strsplit(cfm[2], "\\+")[[1]]) ==  "1")){
+      effectsToCheck <- 2:length(ret$baselearner)
+    }else{
+      effectsToCheck <- 1:length(ret$baselearner)
+    }
+    # predict each effect separately
+    pred <- predict(ret, which=effectsToCheck)
+    
+    # check weather each effect is zero per time-point
+    meanPerTime <- apply(pred, 2, function(x){
+      tapply(x, rep(1:nc, each=nobs), mean) # compute mean per time-point
+    })
+    if(!all(meanPerTime < .Machine$double.eps *10^10)){
+      temp <- colSums(meanPerTime > .Machine$double.eps *10^10)!=0
+      message("The effects ", names(temp)[temp], " do not sum to zero per time-point.")
+    }
+  }
+  
   ### assign new class (e.g. for specialized predictions)
   class(ret) <- c("FDboost", class(ret))
   if(!is.null(id)) class(ret) <- c("FDboostLong", class(ret))
@@ -450,7 +560,8 @@ FDboost <- function(formula,          ### response ~ xvars
   ret$yind <- time
   ret$data <- data
   ret$id <- id
-    
+  attr(ret$id, "nameid") <- nameid
+  
   # if the offset is just an integer the prediction gives back this integer
   ret$predictOffset <- predictOffset
   if(is.null(offsetVec)) ret$predictOffset <- function(time) ret$offset
